@@ -24,6 +24,7 @@
  *
  */
 
+#include <string.h>
 #include <libbonobo.h>
 #include <glib/gmain.h>
 #include <gnome-speech/gnome-speech.h>
@@ -219,7 +220,7 @@ static CORBA_string
 dectalk__get_driverVersion(PortableServer_Servant aservant,
                            CORBA_Environment * ev)
 {
-	return(CORBA_string_dup("0.2"));
+	return(CORBA_string_dup("0.3"));
 }
 
 
@@ -253,7 +254,7 @@ get_voice_list(void)
 	for (i = 0; i < MAXVOICES; i++) {
 		GNOME_Speech_VoiceInfo *info = GNOME_Speech_VoiceInfo__alloc();
 
-		info->language = 0;
+		info->language = CORBA_string_dup("en_US");
 		info->name = CORBA_string_dup(voices[i].description);
 		info->gender = voices[i].gender;
 		l = g_slist_prepend(l, info);
@@ -277,6 +278,43 @@ voice_list_free(GSList *l)
 }
 
 
+static GSList *
+prune_voice_list (GSList *l,
+		  const GNOME_Speech_VoiceInfo *info)
+{
+	GSList *cur, *next;
+	GNOME_Speech_VoiceInfo *i;
+
+	cur = l;
+	while (cur) {
+		i = (GNOME_Speech_VoiceInfo *) cur->data;
+		next = cur->next;
+		if (strlen(info->name))
+			if (strcmp (i->name, info->name)) {
+				CORBA_free (i);
+				l = g_slist_remove_link (l, cur);
+				cur = next;
+				continue;
+			}
+		if (strlen(info->language))
+			if (strcmp (i->language, info->language)) {
+				CORBA_free (i);
+				l = g_slist_remove_link (l, cur);
+				cur = next;
+				continue;
+			}
+		if (info->gender >= 0 && info->gender != i->gender) {
+			CORBA_free (i);
+			l = g_slist_remove_link (l, cur);
+			cur = next;
+			continue;
+		}
+		cur = next;
+	}
+	return l;
+}
+
+
 static GNOME_Speech_VoiceInfoList *
 voice_info_list_from_voice_list(GSList *l)
 {
@@ -296,7 +334,7 @@ voice_info_list_from_voice_list(GSList *l)
 
 		rv->_buffer[i].name = CORBA_string_dup(info->name);
 		rv->_buffer[i].gender = info->gender;
-		rv->_buffer[i].language = info->language;
+		rv->_buffer[i].language = CORBA_string_dup(info->language);
 		i++;
 		l = l->next;
 	}
@@ -352,6 +390,7 @@ dectalk_getVoices(PortableServer_Servant servant,
 	GSList *l;
 
 	l = get_voice_list();
+	l = prune_voice_list (l, voice_spec);
 	rv = voice_info_list_from_voice_list(l);
 	voice_list_free(l);
 
@@ -380,7 +419,17 @@ dectalk_createSpeaker(PortableServer_Servant servant,
                       CORBA_Environment *ev)
 {
 	DectalkSynthesisDriver *d = dectalk_synthesis_driver_from_servant (servant);
-	DectalkSpeaker *s = dectalk_speaker_new(G_OBJECT(d), voice_spec);
+	DectalkSpeaker *s;
+	GSList *l;
+	GNOME_Speech_VoiceInfo *info;
+
+	l = get_voice_list ();
+	l = prune_voice_list (l, voice_spec);
+	if (l)
+		info = l->data;
+	else
+		info = NULL;
+	s = dectalk_speaker_new(G_OBJECT(d), info);
 
 	return(CORBA_Object_duplicate(
 		       bonobo_object_corba_objref(BONOBO_OBJECT(s)), ev));
@@ -566,7 +615,7 @@ main(int argc, char **argv)
 	if (!g_threads_got_initialized)
 		g_thread_init (NULL);
 	
-	obj_id = "OAFIID:GNOME_Speech_SynthesisDriver_Dectalk:proto0.2";
+	obj_id = "OAFIID:GNOME_Speech_SynthesisDriver_Dectalk:proto0.3";
 
 	driver = dectalk_synthesis_driver_new();
 	if (!driver) {
