@@ -68,7 +68,7 @@ select_server (CORBA_Environment *ev)
 	CORBA_Object rv;
   
 	servers = bonobo_activation_query (
-		"repo_ids.has ('IDL:GNOME/Speech/SynthesisDriver:0.2')",
+		"repo_ids.has ('IDL:GNOME/Speech/SynthesisDriver:0.3')",
 		NULL, ev);
 	if (BONOBO_EX (ev)) {
 		return CORBA_OBJECT_NIL;
@@ -106,7 +106,7 @@ select_server (CORBA_Environment *ev)
 static void
 speech_interrupt_test (GNOME_Speech_Speaker s)
 {
-	gint i;
+        gint i, j;
 	gint delay;
 	CORBA_Environment ev;
 	
@@ -114,7 +114,8 @@ speech_interrupt_test (GNOME_Speech_Speaker s)
 
 	for (i = 0; i < 100; i++) {
 		gchar *str = g_strdup_printf ("Test %d\n", i);
-		GNOME_Speech_Speaker_say (s, str, &ev);
+		for (j = 0; j < i; j++)
+		  GNOME_Speech_Speaker_say (s, str, &ev);
 		g_free (str);
 		delay = rand ()%1000000;
 		usleep (delay);
